<?php

namespace Shopware\Components\Api\Resource;

use Shopware\Components\Api\Exception as ApiException;
use Shopware\Models\Order\Detail as OrderDetailModel;
use Shopware\Models\Order\DetailStatus;

/**
 * Class OrderDetail
 *
 * @package Quivers\Components\Api\Resource
 */
class OrderDetail extends Resource
{
     /**
     * @return \Shopware\Models\Order\Repository
     */

    public function getRepository()
    {
        return $this->getManager()->getRepository(OrderDetailModel::class);
    }


    public function getOne($id)
    {
        $this->checkPrivilege('read');

        if (empty($id)) {
            throw new ApiException\ParameterMissingException();
        }

        $builder = $this->getRepository()
            ->createQueryBuilder('s_order_details')
            ->select('s_order_details')
            ->where('s_order_details.id = ?1')
            ->setParameter(1, $id)
            ->addSelect(['attribute'])
            ->leftJoin('s_order_details.attribute', 'attribute');

        /** @var OrderDetailModel $detail */
        $detail = $builder->getQuery()->getOneOrNullResult($this->getResultMode());

        if (!$detail) {
            throw new ApiException\NotFoundException("Detail by id $id not found");
        }
            return $detail;
    }

    /**
     * @param $id
     * @param array $params
     * @return null|object
     * @throws ApiException\ValidationException
     * @throws ApiException\NotFoundException
     * @throws ApiException\ParameterMissingException
     */

     public function update($id, array $params)
    {
        $this->checkPrivilege('update');

        if (empty($id)) {
            throw new ApiException\ParameterMissingException();
        }

        $builder = $this->getRepository()
            ->createQueryBuilder('s_order_details')
            ->select('s_order_details')
            ->where('s_order_details.id = ?1')
            ->setParameter(1, $id)
            ->addSelect(['attribute'])
            ->leftJoin('s_order_details.attribute', 'attribute');

        /** @var OrderDetailModel $detail */
        $detail = $builder->getQuery()->getOneOrNullResult(self::HYDRATE_OBJECT);

        if (!$detail) {
            throw new ApiException\NotFoundException("Order Detail by id $id not found");
        }
        if (isSet($params["statusId"])) {
                /** @var DetailStatus|null $status */
                $status = Shopware()->Models()->find(DetailStatus::class, $params["statusId"]);

                if (!$status) {
                    throw new ApiException\NotFoundException(sprintf(
                        'DetailStatus by id %s not found',
                        $detail['status']
                    ));
                }
                $detail->setStatus($status);
            }
        $order = $detail->getOrder();
        if (isSet($params["invoiceAmount"])){
                $order->setInvoiceAmount($params["invoiceAmount"]);
        }
        $detail->fromArray($params);
        $violations = $this->getManager()->validate($detail);
        if ($violations->count() > 0) {
            throw new ApiException\ValidationException($violations);
        }
        $shippingQuantity = 0;
         $sql = Shopware()->Db()->select()
            ->from('s_order_details_attributes')
            ->where('detailID = '. $id);
        $attributes = Shopware()->Db()->fetchAll($sql);
        foreach ($attributes[0] as $key => $value){
                $subStr = subStr($key, 0, 15);
                if ($subStr == "quantityshipped"){
                    $shippingQuantity = $shippingQuantity + $value;
                }
         }
        foreach ($params["attribute"] as $key => $value){
                $subStr = subStr($key, 0, 15);
                if ($subStr == "quantityshipped"){
                    $shippingQuantity = $shippingQuantity + $value;
                }
         }
        $quantity = $detail->getQuantity();
        $cancellationQuantity = $params["attribute"]["cancellationQuantity"];
        $quantityReturned = $params["attribute"]["quantityReturned"];
        if ($cancellationQuantity == $quantity){
            $status = 2;
        }
        elseif ($quantityReturned == $quantity){
            $status = 4;
        }
        elseif ($shippingQuantity == $quantity){
            $status = 3;
        }
        if (isSet($status)){
            $status = Shopware()->Models()->find(DetailStatus::class, $status);
            $detail->setStatus($status);
        }
        $date = new \DateTime('now', new \DateTimeZone('UTC'));
        $date = ["changed" => $date];
        $order->setAttribute($date);
        $this->flush();

        return $detail;
    }

        /**
     * @param int $offset
     * @param int $limit
     *
     * @return array
     */
    public function getList($offset = 0, $limit = 25, array $criteria = [], array $orderBy = [])
    {
        $this->checkPrivilege('read');

        $builder = $this->getRepository()->createQueryBuilder('s_order_details')
            ->addSelect(['attribute', 'changed', 'attributes'])
            ->leftJoin('s_order_details.order', 'changed')
            ->leftJoin('changed.attribute', 'attributes')
            ->leftJoin('s_order_details.attribute', 'attribute');

        $builder->addFilter($criteria);
        $builder->addOrderBy($orderBy);
        $builder->setFirstResult($offset)
                ->setMaxResults($limit);
        $query = $builder->getQuery();

        $query->setHydrationMode($this->getResultMode());

        $paginator = $this->getManager()->createPaginator($query);

        // Returns the total count of the query
        $totalResult = $paginator->count();

        // Returns the order Detail data
        $orders = $paginator->getIterator()->getArrayCopy();

        return ['data' => $orders, 'total' => $totalResult];
    }
}