<?php
namespace Quivers\Components;

use Quivers\Components\SettingService;
use Shopware\Components\HttpClient\GuzzleFactory;
use Shopware\Components\HttpClient\GuzzleHttpClient;
use Quivers\Components\Logtracking;

class ConfigProvider
{
    protected $session;
    private $settingsService;
    public $factory;
    public $headers;

    public function __construct(SettingService $settingsService, GuzzleFactory $factory, Logtracking $logTracking){
        $this->session = Shopware()->Session();
        $this->settingsService = $settingsService;
        $this->factory = $factory;
        $this->logTracking = $logTracking;
    } 

    public function getConfig(\Enlight_Event_EventArgs $args)
    {
        $view = $args->getSubject()->View();

        $controller = $args->getSubject();

        $request1 = $controller->Request();

        $ViewParams = [];
        $quivers_token = $request1->getParam('payment_token');
        $action = $controller->Request()->getActionName();

        //Get Stripe Id to check if Payment method is Stripe
        $payments = $view->getAssign("sPayments");
        $stripeId = NULL;
        foreach ($payments as $key => $value) {
            if ($value["name"] == "Quivers Stripe"){
                $stripeId =  $value["id"];
            }
        }

        if($request1->getParam('payment_token')){
            $this->session['payment_token'] = json_decode($request1->getParam('payment_token'), true);
        }

        $formData = $view->getAssign("sFormData");
        $userData = $view->getAssign("sUserData");
        
        //fetch billing postcode from userData
        $billingPostCode = $userData["billingaddress"]["zipcode"];

        $captchaKeys = $this->getReCaptchaKeys();
        $view->assign('quiversCaptchaKey', $captchaKeys["site_key"]);    
        $captcha_secret_key = $captchaKeys["site_key_secret"];
        $captcha_response_session = $this->session['stripe_captcha_response'];
        if (($formData["payment"] != $stripeId)){
            //Captcha Will Not be executed
            $captchaExecute = false;
        }
        elseif (!$captcha_response_session["success"] && $stripeId == null && $formData["payment"] == null && ($userData["additional"]["payment"]["name"] == "Quivers Stripe")){
            $this->session['stripe_captcha_token'] = $_POST["g-recaptcha-response"];
            $captcha_response  = $this->executeReCaptcha($_POST["g-recaptcha-response"], $captcha_secret_key);
            $this->session['stripe_captcha_response'] = $captcha_response;
        }
        $public_key = $this->settingsService->get('public_key', 'stripe_settings');
        $ViewParams['publicKey'] = $public_key;
   
        //pass billing postcode to view
        $ViewParams['billing_zipcode'] = $billingPostCode;
   
        if ($request1->getParam('stripeSelectedCard')) {
            $this->session['QuiversStripeSelectedCard'] = json_decode($request1->getParam('stripeSelectedCard'), true);
        }


        $view->QuiversstripePayment = $ViewParams;
    }


    public function executeReCaptcha($captcha_token, $captchaKey){
        if (!$captchaKey){
            return [
                "success" => 1
            ];     
        }
        $captcha_url = 'https://www.google.com/recaptcha/api/siteverify';
        $client = new GuzzleHttpClient($this->factory);
        $captcha_data = [
            "secret" => $captchaKey,
            "response" => $captcha_token,
            "remoteip" => $_SERVER["REMOTE_ADDR"]
        ];
        $this->headers['content-type'] ='application/x-www-form-urlencoded';
        $response = $client->post($captcha_url, $this->headers, $captcha_data);
        $recaptcha_response = json_decode($response->getBody(), true);
        $invalidCodes = $recaptcha_response["error-codes"];
        $allowedViolations = ["invalid-input-secret", "missing-input-response"];
        $this->logTracking->captcha_data($captcha_token, $recaptcha_response["success"]);
        if (count($invalidCodes) == 1 && in_array($invalidCodes[0], $allowedViolations, true)){
            return [
                "success" => 1
            ];
        }
        elseif (count($invalidCodes) == 2) {
            if(in_array($invalidCodes[0], $allowedViolations, true) && in_array($invalidCodes[1], $allowedViolations, true)){
                return [
                    "success" => 1
                ];
            }
        }
        if($recaptcha_response["success"] == true){
            return [
                "success" => 1
            ];
        } else {
            return [
                "success" => 0,
            ];

        }   
    }

    public function getReCaptchaKeys(){
        $quiversSandbox = (bool)$this->settingsService->get('sandbox', 'general');
        $businessRefId = $this->settingsService->get('business_RefId', 'general');
        $baseUrl = 'https://middleware.quiversdemo.com/profile/';
        if(!$quiversSandbox) {
            $baseUrl = 'https://middleware.quivers.com/profile/';
        }
        $url = $baseUrl."retrieve/sitekeys/";
        $request_data = [
            "client_type"=> "Shopware",
            "business_refid"=> $businessRefId
        ];
        $this->headers['content-type'] ='application/json';
        $client = new GuzzleHttpClient($this->factory);
        try{
            $response = $client->post($url, $this->headers, json_encode($request_data));
            $responseDecoded = json_decode($response->getBody(), true);
            return $responseDecoded;
        }catch(\Exception $e){
            return [
                "site_key"=>"",
                "site_key_secret"=> ""
            ];
        }
    }
}