<?php

namespace Quivers\Components;

use Shopware\Components\HttpClient\GuzzleHttpClient as GuzzleClient;
use Shopware\Components\HttpClient\GuzzleFactory;
use Quivers\Components\SettingService;

class MarketplaceConfig 
{
	protected $baseUrl;
	protected $headers;
	protected $client;
	protected $settingsService;

	protected $baseTestUrl = "https://cloudhub.quiversdemo.com/";
	protected $baseProdUrl = "https://cloudhub-internal.quivers.com/";
	protected $marketPlaceApi = "/api/v1/private/ProductGroups/MyProductGroups?refId=";
	protected $claimingPolicyApi = "/api/v1/private/ClaimingPolicies/GetByBusiness?refId=";


	public function __construct(GuzzleFactory $factory, SettingService $settingsService) {
		$this->client = new GuzzleClient($factory);
		$this->settingsService = $settingsService;
	}

    public function getMarketPlaces() {
        
        $quiversAPIKey = $this->settingsService->get('api_key', 'general');
        $quiversSandbox = (bool)$this->settingsService->get('sandbox', 'general');
        $businessRefId = $this->settingsService->get('business_RefId', 'general');

        // get market places by calling the market place api
        if($quiversSandbox) $this->baseUrl = $this->baseTestUrl;
        else $this->baseUrl = $this->baseProdUrl;
        $profileUrl = $this->baseUrl.$this->marketPlaceApi.$businessRefId;
        $this->headers['content-type'] ='application/json';
        $this->headers['Authorization'] ='apikey '.$quiversAPIKey;
        try{
            $response =  $this->client->get($profileUrl, $this->headers);
        }
        catch(\Exception $e){
            return;
        }
        $responseDecoded = json_decode($response->getBody());
        $marketPlaces = [];
        if($response->getStatusCode()==200 && is_array($responseDecoded)) {
            if(count($responseDecoded)>0) {
                foreach($responseDecoded as $res) {
                    if($res->Type=="Marketplace") {
                        array_push($marketPlaces, (object)['id' => $res->Id, 'text' => $res->Name ]);
                    }
                }
            }
        }
        return [
            'marketPlaces'=>$marketPlaces
        ];
    }

	public function getClaimingPolicies() {
        
        $quiversAPIKey = $this->settingsService->get('api_key', 'general');
        $quiversSandbox = (bool)$this->settingsService->get('sandbox', 'general');
        $businessRefId = $this->settingsService->get('business_RefId', 'general');

        // get claiming policies by calling the claiming policy api
        if($quiversSandbox) $this->baseUrl = $this->baseTestUrl;
        else $this->baseUrl = $this->baseProdUrl;
        $claimingPolicyUrl = $this->baseUrl.$this->claimingPolicyApi.$businessRefId;
        $this->headers['content-type'] ='application/json';
        $this->headers['Authorization'] ='apikey '.$quiversAPIKey;
        try{
            $response =  $this->client->get($claimingPolicyUrl, $this->headers);
        }
        catch(\Exception $e){
            return;
        }
        if ($response == null){
            return;
        }
        $responseDecoded = json_decode($response->getBody());
        $claimingPolicy = [];
        if($response->getStatusCode()==200 && is_array($responseDecoded)) {
            if(count($responseDecoded)>0) {
                foreach($responseDecoded as $res) {
                    if ($res->Inclusive == false){
                        continue;
                    }
                    array_push($claimingPolicy, (object)['id' => $res->Id, 'text' => $res->Name ]);
                }
            }
        }
        return [
            'claimingPolicy'=>$claimingPolicy
        ];
    }

}


