<?php

namespace Quivers\Components;

use Shopware\Components\HttpClient\GuzzleFactory;
use Shopware\Components\HttpClient\GuzzleHttpClient;
use Enlight_Controller_ActionEventArgs as ActionEventArgs;
use Quivers\Components\SettingService;
use Quivers\Components\LogTracking;


class TaxCalculator
{

    public $headers;
    public $session;
    public $settingsService;
    public $factory;
    public $logTracking;

    public function __construct(GuzzleFactory $factory, SettingService $settingsService, LogTracking $logTracking)
    {
        $this->factory = $factory;
        $this->logTracking = $logTracking;
        $this->session = Shopware()->Session();
        $this->settingsService = $settingsService;
    }

    public function quivers_tax(ActionEventArgs $args)
    {   
        $startTime = microtime(true);
        $taxEndpoint = '/CustomerOrders/Validate';
        $countriesEndpoint = '/countries';
        $controller = $args->getSubject();
        $view = $controller->View();
        
        $article = $view->getAssign('sBasket');        
        $basketItems = $article["content"];
        $user = $view->getAssign('sUserData');

        $orderVariables = $this->session['sOrderVariables'];

        $quivers_sandbox = (bool)$this->settingsService->get('sandbox', 'general');

        if(!$quivers_sandbox){
            $baseUrl = 'https://api.quivers.com/v1';
        }
        else{
            $baseUrl = 'https://api.quiversdemo.com/v1';
        }

        $taxUrl = $baseUrl.$taxEndpoint;

        # if no item selected then no need to proceed further.
        if(count($article["Quantity"]) <= 0) {
            return $this;
        }

        // No need to calculate tax if user has not specified the shipping address.
        $shipping_address = $user["shippingaddress"];

        if(!$shipping_address) {
            return $this;
        }

        $country = $shipping_address["country"]["id"];
        
        if(!$country) {
            return $this;
        }

        $region = $user["additional"]["stateShipping"]["shortcode"];

        if(!$region){
            $region = 'N/A';
        }

        $currency=$article["sCurrencyName"];

        // Get configured marketplace of order currency.
        $marketPlaceId = $this->getMarketPlaceForCurrency($currency);

        if(!$marketPlaceId) {
            return $this;
        }


        
        $amount = $article["sShippingcosts"];
        $shipping_amt_total = $amount;
        $shipping_amt_per_item = (float) $shipping_amt_total/sizeof($basketItems);
        $shipping_amt_per_item = floatval(substr($shipping_amt_per_item, 0, strpos($shipping_amt_per_item, '.') + 3));
        $shipping_amt_diff = round($shipping_amt_total - ($shipping_amt_per_item * sizeof($basketItems)), 2);

        $itemData = [];

        //BOPIS CODE (check if there is any bopis item)
        $bopis_flag = false;
        foreach ($basketItems as $basketItem) {
            $refId= "";
            if(array_key_exists('__s_order_basket_attributes_businessrefid', $basketItem)){
                $refId = $basketItem['__s_order_basket_attributes_businessrefid'];
                if($refId != ""|| $refId != null){
                    $bopis_flag = true;
                    break;                    
                }
            }
        }

        foreach ($basketItems as $basketItem) {

            $itemId = (int)$basketItem["articleID"];

            $itemName = $basketItem["additional_details"]["articleName"];
            $variantId = (int)$basketItem["articleDetailId"];

            $unitPrice = $basketItem["netprice"];
            
            $bopisShippingFeePerItem = 0;
            $shippingcodeperitem = '';
            if ($bopis_flag == true){
                $bopisShippingFeePerItem = (float)$basketItem['__s_order_basket_attributes_bopis_shipping_fee'];
                $shippingcodeperitem= $basketItem['__s_order_basket_attributes_bopis_shipping_name'];
            }

            $shippingFees = [];
            if($shipping_amt_per_item > 0 && $bopis_flag == false) {
                $shippingFees = [
                    'name' => "FedEx 2-Day",
                    "amount" => $shipping_amt_per_item,
                ];
            }
            elseif($bopisShippingFeePerItem > 0 && $bopis_flag == true){
                $shippingFees = [
                    "name" => $shippingcodeperitem,
                    "amount" => $bopisShippingFeePerItem
                ];
            }

            if(count($shippingFees) > 0 ) {
                $shippingFees = [$shippingFees];
            }

            $taxCode = $basketItem["additional_details"]["product_tax_code"];

            $items = array();
            $items["product"]["name"] = $itemName;
            $items["product"]["refId"] = $itemId;
            $items["product"]["variant"]["id"] = $variantId;
            $items["quantity"] = (int)$basketItem["quantity"];
            $items["pricing"]["unitPrice"] = (float)$unitPrice;
            $items["pricing"]["shippingFees"] = $shippingFees;
            if ($taxCode) $items["product"]["taxCode"] = $taxCode;
           
           array_push($itemData, $items);

        }

        $customer = [
            "firstname"=> $user["additional"]["user"]["firstname"],
            "lastname"=> $user["additional"]["user"]["lastname"],
            "email" => $user["additional"]["user"]["email"]
        ];

        $taxData = [
            "marketplaceId" => $marketPlaceId,
            "customer" => $customer,
            "shippingAddress" => [
                "firstname"=>$shipping_address["firstname"],
                "lastname"=>$shipping_address["lastname"],
                "company"=>!empty($shipping_address["company"]) ? $shipping_address["company"] : "",
                "phoneNumber" => !empty($shipping_address["phone"]) ? $shipping_address["phone"] : "", 
                "line1" => $shipping_address["street"],
                "line2" => !empty($shipping_address["additionalAddressLine1"]) ? $shipping_address["additionalAddressLine1"] : "",
                "city" => $shipping_address["city"],
                "postCode" => $shipping_address["zipcode"],
                "country" => $user["additional"]["countryShipping"]["countryiso"],
                "region" => $region
            ],
            "items" => $itemData
        ];

        if ($bopis_flag == false){
            foreach ($taxData["items"] as &$payload_item) {
                if (!$shipping_amt_diff) {
                    break;
                }

                $payload_item["pricing"]["shippingFees"][0]["amount"] += 0.01;
                $shipping_amt_diff -= 0.01;
            }
            unset($payload_item);
        }
        $this->logTracking->session_start($taxData, date('r', $startTime)."UTC");

        $options = [
            "connect_timeout" => 9.0,
            "timeout" =>10];
        $client = new GuzzleHttpClient($this->factory, $options);
        $this->headers['content-type'] ='application/json';
        try {
            $response = $client->post($taxUrl,  $this->headers, json_encode($taxData));
            $request =  ["tax_url" => $taxUrl, "tax_data" => $taxData];

        } catch (\Exception $e) {
            $this->tax_calculation_via_countries_api($baseUrl, $taxData, $view, $amount);
            return $this;
        }
       
        $responseBody = $response->getBody();

        $totalTax = 0;
        $subtotal = 0;
        $totalAmount = 0;
        $q_tax = [];
        $payment_descriptor = "";
        $includedTax = 0;
        $tax_percent =0;
        if($response->getStatusCode()==200) {

            $responseDecoded = json_decode($responseBody, true);
            $subtotal = $responseDecoded["result"]["totals"]["subtotal"];
            $totalAmount = $responseDecoded["result"]["totals"]["total"];
            $payment_descriptor = $responseDecoded["result"]["statementDescriptor"];

            foreach ($responseDecoded["result"]["items"] as $responseItems) {
   
                $variationRefId = $responseItems['variantId'] == '' ? 0 : $responseItems['variantId'];
                $appliedTaxes = $responseItems['pricing']['taxes'];

                if(count($responseItems['pricing']['taxes']) <= 0) {
                    continue;
                }
           
                foreach ($appliedTaxes as $appliedTax) {
                    if($appliedTax['includedInSubtotal']){
                        $includedTax += $appliedTax["amount"]; 
                    }
                    else{
                        $totalTax += $appliedTax["amount"];
                        $tax_percent = $appliedTax['rate']*100;
                         if(isset($q_tax['q_tax'][$responseItems['productRefId']][$variationRefId])) {
                            $q_tax['q_tax'][$responseItems['productRefId']][$variationRefId] += isset($appliedTax['amount']) && $appliedTax['amount'] > 0 ? $appliedTax['amount'] : 0;
                             $q_tax['q_tax_rate'][$responseItems['productRefId']][$variationRefId] += isset($tax_percent) && $tax_percent > 0 ? $tax_percent : 0;

                        }else  {
                            $q_tax['q_tax'][$responseItems['productRefId']][$variationRefId] = isset($appliedTax['amount']) && $appliedTax['amount'] > 0 ? $appliedTax['amount'] : 0;
                            $q_tax['q_tax_rate'][$responseItems['productRefId']][$variationRefId] = isset($tax_percent) && $tax_percent > 0 ? $tax_percent : 0;

                        }
                    }
                }
            }

            //filter the basket contents
            $orderVariables['sBasket']['content'] = array_map(function($item) use ($q_tax){
                $item['tax'] = $q_tax['q_tax'][$item["articleID"]][$item["articleDetailId"]];
                $item['tax_rate'] = $q_tax['q_tax_rate'][$item["articleID"]][$item["articleDetailId"]];
                return $item;
            }, $orderVariables['sBasket']['content']);
            $orderVariables['sBasketView']['content'] = array_map(function($item) use ($q_tax){
                $item['tax'] = $q_tax['q_tax'][$item["articleID"]][$item["articleDetailId"]];
                $item['tax_rate'] = $q_tax['q_tax_rate'][$item["articleID"]][$item["articleDetailId"]];
                return $item;
            }, $orderVariables['sBasketView']['content']);

            if($totalTax){
                $orderVariables['sBasket']["AmountNumeric"] += $totalTax;
                $orderVariables['sBasket']["AmountNetNumeric"] += $totalTax;
                $orderVariables['sBasket']["AmountWithTaxNumeric"] += $totalTax;
                $orderVariables['sBasket']["sAmountTax"] = $totalTax;

                $orderVariables['sBasketView']["AmountNumeric"] += $totalTax;
                $orderVariables['sBasketView']["AmountNetNumeric"] += $totalTax;
                $orderVariables['sBasketView']["AmountWithTaxNumeric"] += $totalTax;
                $orderVariables['sBasketView']["sAmountTax"] = $totalTax;
            }else{
                $view->assign("includedTax", $includedTax);
            }

            $netAmount = $view->getAssign('sAmount');
            $view->assign('sAmountWithTax', ($netAmount+$totalTax));
            $view->assign('sAmount', ($netAmount+$totalTax));
            $view->assign('additionalTax', $totalTax);

            $paymentKeyFromResponse = $responseDecoded["result"]["payment"];
            if(!empty($paymentKeyFromResponse) && !empty($paymentKeyFromResponse["destination"]))
            {
                $merchantId = !empty($paymentKeyFromResponse["accountId"]) ? $paymentKeyFromResponse["accountId"] : 0;
                $destination = !empty($paymentKeyFromResponse["destination"]) ? $paymentKeyFromResponse["destination"] : "" ;
                $orderVariables["merchantId"] = $merchantId;
                $orderVariables["destination"] = $destination;
            }
            // get payment and set payment_descriptor as additional information
            $orderVariables['payment_descriptor']= $payment_descriptor;
            $endTime = microtime(true);
            $this->logTracking->validate_api_call($request, $responseBody, date('r', $startTime)."UTC", date('r', $endTime)."UTC");
            $order_detail = ["Tax Data"=>$taxData, "Response"=>$responseBody];
            $this->logTracking->session_end($order_detail,date('r', $endTime)."UTC");
        }
        else{
            $this->tax_calculation_via_countries_api($baseUrl, $taxData, $view, $amount);
            return $this;  
        }
          
        
    }
    private function tax_calculation_via_countries_api($baseUrl, $taxData, $view, $shipping_amount) {
        $startTime = microtime(true);
        $article = $view->getAssign('sBasket');        
        $basketItems = $article["content"];
        $countriesEndpoint = '/countries';
        $countriesUrl = $baseUrl.$countriesEndpoint;

        $options = [
            "connect_timeout" => 4.0,
            "timeout" =>5];
        $client = new GuzzleHttpClient($this->factory, $options);
        $this->headers['content-type'] ='application/json';
        try {
            // $response = $this->client->get($countriesUrl,  $this->headers);
            $response = $client->get($countriesUrl,  $this->headers);
        } 
        catch (\Exception $e) {
        }
        if ($response == NULL){
            return;
        }
        if($response->getStatusCode()!=200) {
            return;
        }
        $responseBody = $response->getBody();
        $request = ["base_url" => $baseUrl, "Endpoint"=>$countriesEndpoint];   
        $endTime = microtime(true);
        $this->logTracking->countries_api_call($request, $responseBody, date('r', $startTime)."UTC", date('r', $endTime)."UTC");
        $responsedecoded = json_decode($responseBody, true);
        if (empty($responsedecoded)) {
            return;
        }
        
        $address = $taxData["shippingAddress"];
        $tax_rate = NULL;
        foreach ($responsedecoded['result'] as $country) {
            if ($country['abbreviations']['two']==$address["country"]) {
                foreach ($country["regions"] as $region) {
                    if ($region['abbreviation']==$address["region"]){
                        $tax_rate = $region['maxTaxRate'];
                        break;
                    }
                }
                // Quivers supports "N/A" regions for some countries.
                if (!$tax_rate && array_key_exists('0', $country['regions'])) {
                  $tax_rate = (float) $country['regions']["0"]["maxTaxRate"];
                }
            }
           if ($tax_rate) {
            break;
          }
        }
        if ($tax_rate === NULL) {
            return;
        }
        // calculate and apply worst tax rate.
        $totalTax = 0; // total tax
        $subtotal = 0; // order total excluding tax
        $totalAmount = 0; // order total including tax
        $qqq_data = [];
        $bopisShippingFee = 0.0;
        //BOPIS CODE (check if there is any bopis item)
        $bopis_flag = false;
        foreach ($basketItems as $basketItem) {
            $refId= "";
            if(array_key_exists('__s_order_basket_attributes_businessrefid', $basketItem)){
                $refId = $basketItem['__s_order_basket_attributes_businessrefid'];
                if($refId != ""|| $refId != null){
                    $bopis_flag = true;
                    break;                    
                }
            }
        }

        foreach ($basketItems as $basketItem) {

            $itemQuantity = (int)$basketItem["quantity"];
            $itemUnitPrice = (float)$basketItem["price"]; // item_price
            $itemId = (int)$basketItem["articleID"];
            $variantId = (int)$basketItem["articleDetailId"];
            
            //Bopis Shipping Fee
            $bopisShippingFeePerItem = 0;
            if ($bopis_flag == true){
                $bopisShippingFeePerItem = (float)$basketItem['__s_order_basket_attributes_bopis_shipping_fee'];;
            }
            $bopisShippingFee = $bopisShippingFee +$bopisShippingFeePerItem;

            $itemSubotal = $itemUnitPrice * $itemQuantity;
            $tax_amount = $tax_rate * $itemSubotal;
            $itemTotal = $itemSubotal + $tax_amount; // item_price_with_tax - shipping_fee_per_item
            $qqq_data['q_tax'][$itemId][$variantId] = $tax_amount;
            $qqq_data['q_tax_rate'][$itemId][$variantId] = $tax_rate*100;
            $totalTax = $totalTax + $tax_amount;
            $subtotal = $subtotal + $itemSubotal;
            $totalAmount = $totalAmount + $itemTotal;

        }

        if ($bopis_flag == false){
            $totalAmount = $totalAmount + $shipping_amount;
        }else{
            $totalAmount = $totalAmount + $bopisShippingFee;
        }

        $orderVariables = $this->session['sOrderVariables'];

        //filter the basket contents
        $orderVariables['sBasket']['content'] = array_map(function($item) use ($qqq_data){
            $item['tax'] = $qqq_data['q_tax'][$item["articleID"]][$item["articleDetailId"]];
            $item['tax_rate'] = $qqq_data['q_tax_rate'][$item["articleID"]][$item["articleDetailId"]];
            return $item;
        }, $orderVariables['sBasket']['content']);
        $orderVariables['sBasketView']['content'] = array_map(function($item) use ($qqq_data){
            $item['tax'] = $qqq_data['q_tax'][$item["articleID"]][$item["articleDetailId"]];
            $item['tax_rate'] = $qqq_data['q_tax_rate'][$item["articleID"]][$item["articleDetailId"]];
            return $item;
        }, $orderVariables['sBasketView']['content']);

        //Add tax to order total
        $orderVariables['sBasket']["AmountNumeric"] += $totalTax;
        $orderVariables['sBasket']["AmountNetNumeric"] += $totalTax;
        $orderVariables['sBasket']["AmountWithTaxNumeric"] += $totalTax;
        $orderVariables['sBasket']["sAmountTax"] = $totalTax;

        $orderVariables['sBasketView']["AmountNumeric"] += $totalTax;
        $orderVariables['sBasketView']["AmountNetNumeric"] += $totalTax;
        $orderVariables['sBasketView']["AmountWithTaxNumeric"] += $totalTax;
        $orderVariables['sBasketView']["sAmountTax"] = $totalTax;

        $netAmount = $view->getAssign('sAmount');
        $view->assign('sAmountWithTax', ($totalTax+$netAmount));
        $view->assign('sAmount', ($netAmount+$totalTax));
        $view->assign('additionalTax', $totalTax);    
        $order_detail = ["Tax Data"=>$taxData, "Response"=>$responseBody];
        $this->logTracking->session_end($order_detail, date('r', $endTime)."UTC");
    }   

    public function getMarketPlaceForCurrency($orderCurrency) {
        if($orderCurrency) {
            $marketPlaceId = null;
            switch (strtolower($orderCurrency)) {
                case 'usd':
                    $marketPlaceId = $this->settingsService->get('market_places', 'tax_settings', 'USD');
                    break;
                case 'eur':
                    $marketPlaceId =$this->settingsService->get('market_places', 'tax_settings', 'EUR');
                    break;
                case 'cad':
                    $marketPlaceId = $this->settingsService->get('market_places', 'tax_settings', 'CAD');
                case 'aud':
                    $marketPlaceId = $this->settingsService->get('market_places', 'tax_settings', 'AUD');
                    break;
                case 'jpy':
                    $marketPlaceId = $this->settingsService->get('market_places', 'tax_settings', 'JPY');
                    break;
                case 'gbp':
                    $marketPlaceId = $this->settingsService->get('market_places', 'tax_settings', 'GBP');
                    break;
            }
            return $marketPlaceId;
        }
    } 

}
