<?php

namespace Quivers\Setup;

use Doctrine\DBAL\Connection;
use Shopware\Bundle\AttributeBundle\Service\CrudService;
use Shopware\Components\Model\ModelManager;
use Quivers\Bootstrap\Database;

class Uninstaller
{
    /**
     * @var CrudService
     */
    private $attributeCrudService;

    /**
     * @var ModelManager
     */
    private $modelManager;

    /**
     * @var Connection
     */
    private $connection;

    public function __construct(CrudService $attributeCrudService, ModelManager $modelManager, Connection $connection)
    {
        $this->attributeCrudService = $attributeCrudService;
        $this->modelManager = $modelManager;
        $this->connection = $connection;
    }

    /**
     * @param bool $safeMode
     */
    public function uninstall($safeMode)
    {
        $this->hideAttributes();
        $this->uninstallDatabase();
        $this->deleteReturndedStatus();
    }

    private function hideAttributes()
    {
        $this->attributeCrudService->update('s_order_details_attributes', "trackingCode_1", 'string', [
            'displayInBackend' => false,
        ]);
        $this->attributeCrudService->update('s_order_details_attributes', "quantityShipped_1", 'integer', [
            'displayInBackend' => false,
        ]);
        $this->attributeCrudService->update('s_order_details_attributes', "shippingCarrier_1", 'string', [
            'displayInBackend' => false,
        ]);
        $this->attributeCrudService->update('s_order_details_attributes', "taxRefunded_1", 'float', [
            'displayInBackend' => false,
        ]);
        $this->attributeCrudService->update('s_order_details_attributes', "shippingFee_1", 'float', [
            'displayInBackend' => false,
        ]);
        $this->attributeCrudService->update('s_order_details_attributes', "quantity_returned", 'integer', [
            'displayInBackend' => false,
        ]);
        $this->attributeCrudService->update('s_order_details_attributes', "amount_refunded", 'float', [
            'displayInBackend' => false,
        ]);
        $this->attributeCrudService->update('s_order_details_attributes', "cancellation_quantity", 'integer', [
            'displayInBackend' => false,
        ]);
        $this->attributeCrudService->update('s_articles_attributes', "product_tax_code", 'string', [
            'displayInBackend' => false,
        ]);
        $this->attributeCrudService->update('s_articles_attributes', "quivers_preorder", 'boolean', [
            'displayInBackend' => false,
        ]);
        $this->attributeCrudService->update('s_articles_attributes', "quivers_upc", 'string', [
            'displayInBackend' => false,
        ]);
        $this->modelManager->generateAttributeModels(['s_order_details_attributes', 's_articles_attributes']);

    }

    private function uninstallDatabase()
    {
    	$database = new Database($this->modelManager);
        $database->uninstall();
    }


    public function deleteReturndedStatus (){
        $data = [
            "description" => "Returned"
        ];
        $this->connection->delete("s_core_detail_states", $data);
        $data = [
            "name" => "Returned",
        ];
        $this->connection->delete("s_core_states", $data);             
    }

}

